
// The following ifdef block is the standard way of creating macros which make exporting 
// from a DLL simpler. All files within this DLL are compiled with the LINMODEL_EXPORTS
// symbol defined on the command line. this symbol should not be defined on any project
// that uses this DLL. This way any other project whose source files include this file see 
// LINMODEL_API functions as being imported from a DLL, wheras this DLL sees symbols
// defined with this macro as being exported.
#ifdef LINMODEL_EXPORTS
#define LINMODEL_API __declspec(dllexport)
#else
#define LINMODEL_API __declspec(dllimport)
#endif

LINMODEL_API int lin_model(const int npts, const int npar, const int ncon, double * w, double * y, double ** x, double * par, double ** mat, double ** c, double &t, double &wt);
// npts is the number of experimental points: must be >npar
// npar is the number of independent parameters: must be >0
// ncon is the number of constraints: must be >=0
// on entry w contains the weights of experimental points (unchanged on return)
// on entry y contains observed values, on return it contains residuals
// on return par (must have length npar+ncon) contains the parameters of the linear model and the Lagrange multipliers associated with the constraints
// on return t contains the "residual mean square" (minimised by this routine)
// mat is a square matrix (dimension npar+ncon) which will be used by lin_predict to calculate the variance in the prediction
// on return wt contains the sum of the weights of experimental points

// constraints are:  sum_i par_i*c_il = d_l for l=1...ncon
// on entry d_l must be supplied in par[npar] through par[npar+ncon-1]

// IMPORTANT: par, mat and t (as returned by lin_model) must not be changed as they are used by lin_predict


LINMODEL_API void lin_predict(const int npar, double * x, double * par, double ** mat, double t, double &value, double &var);
// x is the vector of independent quantities for which the model prediction is sought
// par, mat and t must have been previously calculated by lin_model and not been changed
// value is the predicted model value; var is the variance (square of standard deviation) of value

// the return value of lin_model is normally 0, indicating success
// if a problem is detected, it returns one of the values defined below
const int LINMODEL_ERR_Singular_data = 1;
const int LINMODEL_ERR_Invalid_number_of_constraints = 2;
const int LINMODEL_ERR_Invalid_number_of_parameters = 3;
const int LINMODEL_ERR_Insufficient_data = 4;
const int LINMODEL_ERR_Too_many_constraints = 5;
