#include <stdio.h>
#include <math.h>
#include "linmodel.h"

void main(int argc, char **argv) {
	int i,j;

//	for (i=0;i<argc;i++) {
//		printf("%d: %s\n",i,argv[i]);
//	}

	if (argc>1) {
		FILE *input;
		int npt,npr,nc;
		input=fopen(argv[1],"r");
		if (input==NULL) {
			printf("Error in opening %s\n",argv[1]);
		}
		else
		{
			double *w,*y,**x,*par,t,**c,**mat,value,sdev,ss=0,wt,*xt;
			fscanf(input,"%d %d %d",&npt,&npr,&nc);
			printf("npt = %d, npr = %d, nc = %d\n",npt,npr,nc);

	// allocate memory for the vectors and matrices needed
			w=new double[npt];
			y=new double[npt];
			par=new double[npr+nc];
			mat=new double *[npr+nc];
			for (i=0;i<npr+nc;i++) {
				mat[i]=new double[npr+nc];
			}
			x=new double *[npt];
			for (i=0;i<npt;i++) {
				x[i]=new double[npr];
			}
			c=new double *[nc];
			for (i=0;i<nc;i++) {
				c[i]=new double[npr];
			}
			xt=new double[npr+nc];

	// read (and print) datapoints
			for (i=0;i<npt;i++) {
				fscanf(input,"%lf %lf",w+i,y+i);
				if (w[i]<0) w[i]=0;
				printf("%d: w = %lf  y = %lf",i,w[i],y[i]);
				ss += w[i]*y[i]*y[i];
				for (j=0;j<npr;j++) {
					fscanf(input,"%lf",x[i]+j);
					printf("  x(%d) = %lf",j,x[i][j]);
				}
				printf("\n");
			}
			printf("ss = %lf\n",ss);

	// read (and print) constraints
			for (i=0;i<nc;i++) {
				fscanf(input,"%lf",par+i+npr);
				printf("%d: d = %lf",i,par[i+npr]);
				for (j=0;j<npr;j++) {
					fscanf(input,"%lf",c[i]+j);
					printf("  c(%d) = %lf",j,c[i][j]);
				}
				printf("\n");
			}

	// do the regression calculation
			i=lin_model(npt,npr,nc, w, y, x, par, mat, c, t, wt);
			switch (i) {
			case LINMODEL_ERR_Singular_data:
				printf("No solution possible with these data\n");
				break;
			case LINMODEL_ERR_Invalid_number_of_constraints:
				printf("You specified a negative number of comstraints\n");
				break;
			case LINMODEL_ERR_Invalid_number_of_parameters:
				printf("The number of parameters must be positive\n");
				break;
			case LINMODEL_ERR_Insufficient_data:
				printf("There must be more datapoints than parameters\n");
				break;
			case LINMODEL_ERR_Too_many_constraints:
				printf("There must be fewer constraints than parameters\n");
				break;
			default:
	// retrieve and print model parameters and their uncertainties
				for (j=0;j<npr;j++) {
					for (i=0;i<npr;i++) {
						xt[i]=0;
					}
					xt[j]=1;
					lin_predict(npr, xt, par, mat, t, value, sdev);
					printf("  par(%d) = %.9lf %.9lf\n",j,value,sqrt(sdev));
				}
	// print model statistics
				printf("R-sq = %lf\n",1-t*(wt-npr)/ss);
				ss -= t*(wt-npr);
				printf("Reg. MS = %lf F = %lf\n",ss/npr,ss/(npr*t));
				printf("Res. MS = %.9lf  Res.Stdev. = %.9lf\n",t,sqrt(t));

	// apply the model on its own data
				for (i=0;i<npt;i++) {
					lin_predict(npr, x[i], par, mat, t, value, sdev);
					printf("%d: y = %lf %lf %lf %lf",i,y[i],value,sdev,sqrt(sdev));
					printf("\n");
				}
			}
			delete [] xt;
			delete [] c;
			delete [] x;
			delete [] mat;
			delete [] par;
			delete [] y;
			delete [] w;
		}
	}
}
